#!/usr/bin/env python
#
""" dns_setup
Run this on a yaml dns definition an it will check the cloud to make sure your dns is setup
and if not will set it up.
"""

import logging
from optparse import OptionParser
import yaml
import sys

import boto

from cirrus.r53 import Zone

log = logging.getLogger('cirrus')
log.addHandler(logging.StreamHandler())

def get_args():
    """Sets up Option parser and then resturns the parsed options and args."""
    usage = "usage: %prog [options] <yaml definition>"
    parser = OptionParser(usage=usage)
    parser.add_option('-d', '--dry-run', action='store_true', dest='dry_run', default=False, \
        help="Report what would be done but do nothing.")
    parser.add_option('-s', '--show', action='store_true', dest='show', default=False, \
        help="Returns a bind style zone file for the defined domains.")
    parser.add_option('--terminate', action='store_true', dest='terminate', default=False, \
        help="Instead of creating zones delete them.")
    parser.add_option('-v', '--verbose', action='store_true', dest='verbose', default=True)

    return parser.parse_args()

def main():
    options, args = get_args()

    if options.verbose:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.WARN)

    #Pull from the yaml file
    def_file = open(args[0], 'r')
    dns_def = yaml.load(def_file)
    zones = dns_def['zones']

    #Get the connection
    conn = boto.connect_route53(dns_def['access_id'], dns_def['secret_key'])

    if options.dry_run:
        log.info("Doing a dry-run, only reporting actions.")

    for name, zone_file in zones.iteritems():
        r53zone = Zone(conn, name)
       
        if r53zone.exists():
            if options.terminate:
                r53zone.remove(options.dry_run)
            elif options.show:
                log.info(str(r53zone))
            else:
                r53zone.update(zone_file, options.dry_run)
        elif options.show or options.terminate:
            log.info('Zone %s does not exist' % (name))
        else:
            r53zone.create(zone_file, options.dry_run)

if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/env python
#

import logging
import time

#Filter out the pycrypto(used by paramiko) deprecation warnings. See https://bugs.launchpad.net/paramiko/+bug/271791
import warnings
from Crypto.pct_warnings import RandomPool_DeprecationWarning
warnings.filterwarnings('ignore', category=RandomPool_DeprecationWarning)
import paramiko

log = logging.getLogger('cirrus')

class ElasticCompute:
    """ Elastic Compute Cloud object of cirrus
    This object takes a boto ec2 connection and ec2 definitions and provides
    methods to perform operations on ec2 infrastructure.
    """
    
    def __init__(self, conn, base, defined_nodes):
        self.conn = conn #Boto EC2 connection
        self.base = base
        self.nodes = defined_nodes

    def create_instance(self, node_name):
        """Creates a new instance based on the node definition. Returns the instance."""
        image = self.conn.get_image(self.nodes['image'])
        reservation = image.run(key_name=self.nodes['key']['name'], 
        security_groups=self.nodes['security_groups'], instance_type=self.nodes['type'], \
            monitoring_enabled=self.nodes['monitoring'])
        log.debug('reservation ' + str(reservation))
        inst = reservation.instances[0]
        log.debug('inst ' + inst.id)

        #Make sure it starts before doing anything else
        while inst.state != 'running':
            log.debug('Waiting for instance to start up, current state ' + inst.state)
            time.sleep(5)
            inst.update()

        #Assign the name
        self.conn.create_tags([inst.id], {'Name':node_name})

        #Run bootstrap commands
        if self.nodes.has_key('deploy_cmds'):
            #Sleep for a minute before trying this
            time.sleep(60)
            for cmd in self.nodes['deploy_cmds']:
                self.ssh_cmd(cmd, self.nodes['key']['user'], 
                inst.public_dns_name, self.nodes['key']['file'])

        return inst

    def create_instances(self, dry_run=False):
        """Create defined instances that don't exist.
        If dry_run is set just log the action but don't perform it.
        """
        instances = {}
        names = self.get_instance_names()
        for name in self.nodes['names']:
            if name not in names:
                log.info('Starting ' + name)
                if not dry_run:
                    instances[name] = self.create_instance(name).id
            else:
                inst = self.get_instance(name)
                instances[name] = inst.id
                if inst.state != 'running' and inst.state != 'pending':
                    if not dry_run:
                        inst.start()
                    if inst.public_dns_name is not None:
                        log.info('Instance ' + name + " exists but isn't running, starting.\n\tPublic dns:" \
                            + inst.public_dns_name)
                    else:
                        log.info('Instance ' + name + " exists but isn't running, starting. Name not yet known.")
                else:
                    log.info('Instance ' + name + ' exists and has public dns name: ' + inst.public_dns_name)

        return instances

    def create_key_pair(self, dry_run=False):
        """Creates a key pair if it doesn't already exist.
        Returns the key object."""
        key_name = self.nodes['key']['name']
        key_file = self.nodes['key']['file']
        key = self.conn.get_key_pair(key_name)
        if key is None:
            log.info('Creating key pair ' + key_name)
            if not dry_run:
                key = conn.create_key_pair(key_name)
                if os.path.exists(key_file):
                    log.error('Key file ' + key_file + " already exists!/nNot saving newly create key " + key_name)
                else:
                    key.save(key_file)

        return key

    def get_instance(self, name):
        """Given a name find the instance.
        I assume names are unique for instances."""
        return self.conn.get_all_instances(filters={'tag:Name':name})[0].instances[0]

    def get_instance_names(self):
        """Return a list of named instances."""
        instances = []
        for r in self.conn.get_all_instances():
            instances.extend(r.instances) 
        #The above code finds all instances, below only the named ones
        tags = self.conn.get_all_tags(filters={'key':'Name'})
        names = []
        for tag in tags:
            if tag.res_id[0] == 'i': #only instances not anything else with a tag
                names.append(tag.value)
        log.debug('Found names ' + str(names))
        if len(instances) != len(names):
            log.warning("There are %d instances running, %d are named" % (len(instances), len(names)) + \
            "\nThis script only deals with named instances.")
        return names

    @staticmethod
    def ssh_cmd(cmd, user, host, key_file):
        """Run the ssh command on the given host, logging in with the specified key."""
        log.debug('Running command %s on %s as %s' % (cmd, host, user))
        ssh = paramiko.SSHClient()
        ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
        try:
            ssh.connect(hostname=host, username=user, pkey=paramiko.RSAKey(filename=key_file))
            stdin, stdout, stderr = ssh.exec_command(cmd)
        except Exception, msg:
            log.error("ssh command %s caused a python exception:\n%s" % (cmd, str(msg)))
        else:
            out = stdout.read()
            err = stderr.read()
            if len(err) > 0:
                log.error("Command %s exited with stdout:\n%s\nstderr:\n%s" % (cmd, out, err))

    def stop_instances(self, terminate=False, dry_run=False):
        """Stop defined instances that exist.
        If dry_run is set just log the action but don't perform it.
        If terminate is defined the nodes are terminated not stopped.
        """
        names = self.get_instance_names()
        for name in self.nodes['names']:
            if name in names:
                if not dry_run:
                    inst = self.get_instance(name)
                    if inst.state != 'stopped':
                        if terminate:
                            log.info('Terminating ' + name)
                            inst.terminate()
                        else:
                            log.info('Stopping ' + name)
                            inst.stop()



#!/usr/bin/env python
#
""" ec2_setup
Run this on a yaml cloud definition an it will check the cloud to make sure your ec2 definition is setup
and if not will set it up.
"""

import logging
from optparse import OptionParser
import sys
import yaml

import boto

from cirrus.ec2 import ElasticCompute
from cirrus.elb import LoadBalance

log = logging.getLogger('cirrus')
log.addHandler(logging.StreamHandler())

def create_security_groups(conn, base, dry_run=False):
    """Create security groups which don't already exist.
    It does not modify existing groups.
    """
    groups = conn.get_all_security_groups()
    group_names = [ g.name for g in groups ]
    for name, group in base['security_groups'].iteritems():
        log.debug('Checking security group ' + name)
        if name not in group_names:
            log.info('Creating security group ' + name)
            if not dry_run:
                new_group = conn.create_security_group(name, group['description'])
                for rule in group['authorize']:
                    log.info("\tAdding rule " + str(rule))
                    if rule.has_key('src_group'):
                        rule['src_group'] = conn.get_all_security_groups([rule['src_group'], ])[0]
                    new_group.authorize(**rule)

def get_args():
    """Sets up Option parser and then resturns the parsed options and args."""
    usage = "usage: %prog [options] <yaml definition>"
    parser = OptionParser(usage=usage)
    parser.add_option('-d', '--dry-run', action='store_true', dest='dry_run', default=False, \
        help="Report what would be done but do nothing.")
    parser.add_option('-v', '--verbose', action='store_true', dest='verbose', default=True)
    parser.add_option('-s', '--stop', action='store_true', dest='stop', default=False, \
        help="Instead of starting up defined instances stop them. Stop does not destroy EBS volumes or load balancers.")
    parser.add_option('--terminate', action='store_true', dest='terminate', default=False, \
        help="Instead of starting up defined instances terminiate them. This will also remove load balancers.")

    return parser.parse_args()

def main():
    options, args = get_args()

    if options.verbose:
        log.setLevel(logging.DEBUG)
    else:
        log.setLevel(logging.WARN)

    #Pull from the yaml file
    def_file = open(args[0], 'r')
    definition = yaml.load_all(def_file)
    base = definition.next()

    if options.dry_run:
        log.info("Doing a dry-run, only reporting actions.")

    #Get the api connection
    conn = boto.connect_ec2(base['access_id'], base['secret_key'])

    #process security groups
    create_security_groups(conn, base, options.dry_run)

    #Run through the ec2 definitions for each type, collect instances
    instances = {}
    for defined in definition:
        ec2 = ElasticCompute(conn, base, defined)

        #process key pair
        ec2.create_key_pair(options.dry_run)

        #process instances
        if options.stop or options.terminate:
            ec2.stop_instances(options.terminate, options.dry_run)
        else:
            new_instances = ec2.create_instances(options.dry_run)
            instances.update(new_instances)

    #Process load balancing
    elb = LoadBalance(boto.connect_elb(base['access_id'], base['secret_key']), base, instances)
    if options.terminate:
        elb.remove_load_balancers(options.dry_run)
    else:
        elb.create_load_balancers(options.dry_run)

if __name__ == "__main__":
    sys.exit(main())



import os
from setuptools import find_packages, setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

setup(
    name = "Cirrus",
    version = "0.1",
    author = "Tim Kuhlman",
    author_email = "tim@backgroundprocess.com",
    description = ("Define and setup a cloud infrastructure, leveraging libcloud and yaml."),
    license = "BSD",
    keywords = "cloud libcloud yaml ec2 route53",
    include_package_data = True,
    url = "http://packages.python.org/cirrus",
    packages=find_packages(),
    scripts=['bin/dns_setup.py', 'bin/ec2_setup.py'],
    long_description=read('README.txt'),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
    install_requires = ['boto', 'PyYAML>=3.09', 'paramiko>=1.7.6', 'dnspython'],
)




#!/usr/bin/env python
#

import logging

import boto

log = logging.getLogger('cirrus')

class LoadBalance:
    """ Elastic Load Balancing object of cirrus
    This object takes a boto elb connection and a cirrus ec2 object and 
    has methods to perform operations on elb's.
    """

    def __init__(self, conn, base, instances):
        self.base = base
        self.conn = conn #A boto elb connection
        self.instances = instances #{instance name: instance id} of defined instances

    def create_load_balancers(self, dry_run=False):
        """Create load_balancers which don't already exist.
        It does not modify existing groups.
        """
        if self.base['load_balancers'] is None:
            return None

        balancers = {}
        for b in self.conn.get_all_load_balancers():
            balancers[b.name] = b
            
        #Check defined balancers to see if they exists
        for name, bsetup in self.base['load_balancers'].iteritems():
            log.debug('Checking load balancer ' + name)
            if name not in balancers.keys():
                log.info('Creating Load balancer ' + name)
                #create a new balancer
                listeners = [ (l['balancer_port'], l['instance_port'], l['protocol']) for l in bsetup['listeners'] ]
                if not dry_run:
                    lb = self.conn.create_load_balancer(name, bsetup['zones'], listeners)

                    #Add health check
                    lb.configure_health_check(boto.ec2.elb.HealthCheck(**bsetup['check']))

                    #add machines to the lb
                    instance_ids = [ self.instances[name] for name in bsetup['instances'] ]
                    lb.register_instances(instance_ids)
            else:
                log.info('Load balancer ' + name + ' already exists, dns name = ' + balancers[name].dns_name)
                #check that all instances are setup, extra instances are ignored
                for instance_name in bsetup['instances']:
                    instance_id = self.instances[instance_name]
                    if instance_id not in balancers[name].instances:
                        log.info('Load balancer ' + name + ' was missing instance ' + instance_name + ' adding.')
                        if not dry_run:
                            balancers[name].register_instances([instance_id, ])

    def remove_load_balancers(self, dry_run=False):
        """Remove defined load_balancers which exist.
        """
        balancers = self.conn.get_all_load_balancers()
        defined_names = [ b.name for b in self.base['load_balancers'].iterkeys() ]
        #Check defined balancers to see if they exists
        for balancer in balancers:
            if balancer.name in defined_names:
                log.info('Removing load balancer ' + balancer.name)
                if not dry_run:
                    balancer.delete()




#!/usr/bin/env python
#

import logging
import time
import tempfile

from lxml import etree
import dns
import dns.zone

log = logging.getLogger('cirrus')

class Zone:
    """ An interface to Amazon web services route 53.
    The class defines an aws zone and can create the resource records,
    or it can compare the existing Resource Records to a bind style zone file and update.
    The resource records must be sent to amazon as a xml file, the format can be found at.
    http://docs.amazonwebservices.com/Route53/latest/APIReference/index.html?API_ChangeResourceRecordSets.html
    """

    def __init__(self, conn, zone_name):
        self.conn = conn
        self.zone_name = zone_name
        #Set on create or exists call
        self.id = None

    def __repr__(self):
        """Return a bind style zone file for the current zone in aws."""
        if not self.exists():
            return self.zone_name + " does not exist."

        dnszone = self._to_dnszone()
        tmp = tempfile.TemporaryFile()
        dnszone.to_file(tmp)
        tmp.seek(0)
        return tmp.read()

    def _change_xml(self, action, name, rtype, ttl, values):
        """Returns amazon Change xml for adding an entry."""
        xmlout = "   <Change>\n" + "    <Action>" + action + "</Action>\n" + "     <ResourceRecordSet>\n" + \
            "      <Name>" + name + "</Name>\n" + "      <Type>" + rtype + "</Type>\n" + \
            "      <TTL>" + str(ttl) + "</TTL>\n" + "      <ResourceRecords>\n"

        for rvalue in values:
            xmlout += "       <ResourceRecord>\n" + "        <Value>" + rvalue + "</Value>\n" + \
            "       </ResourceRecord>\n" 
            
        xmlout += "     </ResourceRecords>\n" + "    </ResourceRecordSet>\n" + "   </Change>\n"
        return xmlout

    def _add_change_xml(self, name, rtype, ttl, values):
        """Returns amazon Change xml for adding an entry."""
        return self._change_xml('CREATE', name, rtype, ttl, values)

    def _delete_change_xml(self, name, rtype, ttl, values):
        """Returns amazon Change xml for deleting an entry."""
        return self._change_xml('DELETE', name, rtype, ttl, values)

    def _update_change_xml(self, name, rtype, ttl, old_values, new_values):
        """Returns amazon Change xml for updating an entry."""
        xmlout = self._change_xml('DELETE', name, rtype, ttl, old_values)
        xmlout += self._change_xml('CREATE', name, rtype, ttl, new_values)
        return xmlout

    def _create_changeset(self, adds, deletes, updates):
        """Create amazon changeset xml for adds and deletes and updates to a zone.
        """
        if len(adds) == 0 and len(deletes) == 0 and len(updates) == 0:
            return None

        #Header
        xmlout = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + \
            "<ChangeResourceRecordSetsRequest xmlns=\"https://route53.amazonaws.com/doc/2010-10-01/\">\n" + \
            " <ChangeBatch>\n" + "  <Comment>Create Zone " + self.zone_name + "</Comment>\n" + "  <Changes>\n"

        for key, value in adds.iteritems():
            xmlout += self._add_change_xml(key[0], key[1], key[2], value)

        for key, value in updates.iteritems():
            xmlout += self._update_change_xml(key[0], key[1], key[2], value[0], value[1])

        for key, value in deletes.iteritems():
            xmlout += self._delete_change_xml(key[0], key[1], key[2], value)

        #footer
        xmlout += "  </Changes>\n" + " </ChangeBatch>\n" + "</ChangeResourceRecordSetsRequest>\n"
        return xmlout

    def _compare(self, from_zone, to_zone):
        """Compare two dns zones and return resource record dictionaries for add deletes and updates.
        Ignores SOA and root NS records because Amazon autogenerates those."""
        from_records = self._get_rrecords(from_zone)
        to_records = self._get_rrecords(to_zone)
        adds = {}
        deletes = {}
        updates = {}

        for key in to_records.iterkeys(): #key is [name, rtype, ttl]
            name = key[0]
            rtype = key[1]
            #skip records amazon automatically generates
            if rtype == 'NS' and name[:-1] == self.zone_name:
                continue
            elif rtype == 'SOA':
                continue
            if from_records.has_key(key):
                from_records[key].sort() #I sort both to avoid ordering issues
                to_records[key].sort()
                if from_records[key] != to_records[key]:
                    log.debug("Updating %s %s %s to %s" % (key[0], key[1], key[2], to_records[key]))
                    updates[key] = [from_records[key], to_records[key]]
                del from_records[key] #no-op or modify either way pull from from_records.
            else:
                adds[key] = to_records[key]
                log.debug("Adding %s %s %s %s" % (key[0], key[1], key[2], to_records[key]))

        #Anything remaining in the from_records is a delete
        for key in from_records.iterkeys():
            deletes[key] = from_records[key]
            log.debug("Removing %s %s %s %s" % (key[0], key[1], key[2], from_records[key]))

        return adds, deletes, updates

    def _create_xml(self, zone_file):
        """Create a Amazon change resource record xml file given a bind style zone file."""
        dnszone = dns.zone.from_file(zone_file, origin=self.zone_name, relativize=False)
        xmlout = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + \
            "<ChangeResourceRecordSetsRequest xmlns=\"https://route53.amazonaws.com/doc/2010-10-01/\">\n" + \
            " <ChangeBatch>\n" + "  <Comment>Create Zone " + self.zone_name + "</Comment>\n" + "  <Changes>\n"

        rrecords = self._get_rrecords(dnszone)
        for key in rrecords.iterkeys():
            xmlout += self._add_change_xml(key[0], key[1], key[2], rrecords[key])

        xmlout += "  </Changes>\n" + " </ChangeBatch>\n" + "</ChangeResourceRecordSetsRequest>\n"
        return xmlout
    
    def _get_rrecords(self, dnszone):
        """Given a dns zone return a dictionary of rrecords, with a format
        {(name, rtype, ttl): rvalue} where each variable is a string.
        Skips any SOA entries and NS entries for the root.
        """
        rrecords = {}
        for name, ttl, rdata in dnszone.iterate_rdatas():
            rtype = dns.rdatatype.to_text(rdata.rdtype)
            name = str(name)
            if rtype == 'NS' and name[:-1] == self.zone_name:
                continue
            elif rtype == 'SOA':
                continue

            rvalue = rdata.to_text()
            if (name, rtype, ttl) in rrecords:
                new_value = rrecords[(name, rtype, ttl)]
                new_value.append(rvalue)
                rrecords[(name, rtype, ttl)] = new_value
            else:
                rrecords[(name, rtype, ttl)] = [rvalue]

        return rrecords

    def _to_dnszone(self):
        """Gets all resource records from route 53 and parses them into a dns.zone object."""
        xml = etree.fromstring(self.conn.get_all_rrsets(self.id))
        ns = xml.tag.split('}')[0] + '}'

        simple_bind = ""
        for rrecord in xml.findall(ns + 'ResourceRecordSets/' + ns + 'ResourceRecordSet'):
            rtype = rrecord.find(ns + 'Type').text
            name = rrecord.find(ns + 'Name').text
            ttl = rrecord.find(ns + 'TTL').text
            simple_bind += "$TTL %s\n" % (ttl)

            for value in rrecord.findall('%sResourceRecords/%sResourceRecord/%sValue' % (ns, ns, ns)):
                simple_bind += "%s\tIN\t%s\t%s\n" % (name, rtype, value.text)

        zone = dns.zone.from_text(simple_bind, origin=self.zone_name, relativize=False)
        return zone

    def create(self, zone_file, dry_run):
        """ Create the zone and populate with settings from the passed in zone_file.
        Do nothing, report only, if dry_run is true.
        """
        if self.id != None:
            log.error('A zone definition already exists for zone ' + self.zone_name)
            return
        
        log.info('Creating zone ' + self.zone_name)
        if not dry_run:
            zone = self.conn.create_hosted_zone(self.zone_name)['CreateHostedZoneResponse']
            nameservers = ""
            for nameserve in zone['DelegationSet']['NameServers']:
                nameservers += nameserve
            log.info('Zone nameservers: ' + nameservers)
            self.id = zone['HostedZone']['Id'].replace('/hostedzone/', '')
            zone_xml = self._create_xml(zone_file)
            #log.debug(zone_xml)
            log.info("Adding rrsets to zone " + self.zone_name)
            change = self.conn.change_rrsets(self.id, zone_xml)

    def exists(self):
        """Return true if the self.zone_name exists on AWS, false otherwise."""
        if self.id != None:
            return True
        zones = self.conn.get_all_hosted_zones().values()[0]['HostedZones']
        for zone in zones:
            if zone['Name'][:-1] == self.zone_name: #strip the trailing .
                self.id = zone['Id'].replace('/hostedzone/', '')
                return True

        return False

    def update(self, zone_file, dry_run):
        """Compare existing Resource Records to the given zone file and update if needed.
        Do nothing, report only, if dry_run is true.
        """
        if self.id == None:
            if not self.exists():
                log.error('The zone ' + self.zone_name + " doesn't exist, create it don't update.")
                return

        dnszone = dns.zone.from_file(zone_file, origin=self.zone_name, relativize=False)
        r53zone = self._to_dnszone()

        if dnszone == r53zone: #This compares SOA and root NS records, which are ignored in actual updates
            log.info('Zone %s is up to date.' % self.zone_name)
            return
    
        adds, deletes, updates = self._compare(r53zone, dnszone)
        changeset = self._create_changeset(adds, deletes, updates)
        if changeset is None:
            log.info("No differences found for zone %s" % self.zone_name)
            return

        log.info('Updating zone %s' % self.zone_name)
        if not dry_run:
            change = self.conn.change_rrsets(self.id, changeset)

    def remove(self, dry_run):
        """Remove this zone from AWS."""
        if self.id == None:
            if not self.exists():
                log.info("Zone " + self.zone_name + " doesn't exist.")
                return

        log.info("Removing zone " + self.zone_name)
        if not dry_run:
            change_needed = False
            xml = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + \
            "<ChangeResourceRecordSetsRequest xmlns=\"https://route53.amazonaws.com/doc/2010-10-01/\">\n" + \
            " <ChangeBatch>\n" + "  <Comment>Delete Zone " + self.zone_name + "</Comment>\n" + "  <Changes>\n"

            rrecords = etree.fromstring(self.conn.get_all_rrsets(self.id))
            ns = rrecords.tag.split('}')[0] + '}'
            for rrecord in rrecords.findall(ns + 'ResourceRecordSets/' + ns + 'ResourceRecordSet'):
                rtype = rrecord.find(ns + 'Type').text
                name = rrecord.find(ns + 'Name').text
                if rtype == 'NS' and name[:-1] == self.zone_name:
                    continue
                elif rtype == 'SOA':
                    continue
                change_needed = True
                xml += "   <Change>\n" + "    <Action>DELETE</Action>\n" + \
                    etree.tostring(rrecord, pretty_print=True) + "   </Change>\n"

            xml += "  </Changes>\n" + " </ChangeBatch>\n" + "</ChangeResourceRecordSetsRequest>\n"
            if change_needed:
                change = self.conn.change_rrsets(self.id, xml)
            delete = self.conn.delete_hosted_zone(self.id)
